@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.25 2004/03/19 21:46:34 dolorous Exp $ */
+/* $Id: files.c,v 1.37 2004/06/01 20:37:59 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -105,7 +105,8 @@ void new_file(void)
 #endif
 }
 
-filestruct *read_line(char *buf, filestruct *prev, int *line1ins, int len)
+filestruct *read_line(char *buf, filestruct *prev, int *line1ins, size_t
+	len)
 {
     filestruct *fileptr = (filestruct *)nmalloc(sizeof(filestruct));
 
@@ -189,7 +190,7 @@ int read_file(FILE *f, const char *filename, int quiet)
 	   assume it's a DOS or Mac formatted file if it hasn't been
 	   detected as one already! */
 	if (fileformat == 0 && !ISSET(NO_CONVERT)
-		&& is_cntrl_char((int)input) != 0 && input != '\t'
+		&& is_cntrl_char(input) && input != '\t'
 		&& input != '\r' && input != '\n')
 	    SET(NO_CONVERT);
 #endif
@@ -785,17 +786,14 @@ int add_open_file(int update)
     /* if we're updating, save current modification status (and marking
        status, if available) */
     if (update) {
-#ifndef NANO_SMALL
 	if (ISSET(MODIFIED))
 	    open_files->file_flags |= MODIFIED;
+#ifndef NANO_SMALL
 	if (ISSET(MARK_ISSET)) {
 	    open_files->file_mark_beginbuf = mark_beginbuf;
 	    open_files->file_mark_beginx = mark_beginx;
 	    open_files->file_flags |= MARK_ISSET;
 	}
-#else
-	if (ISSET(MODIFIED))
-	    open_files->file_flags |= MODIFIED;
 #endif
     }
 
@@ -1397,8 +1395,8 @@ int copy_file(FILE *inn, FILE *out)
  * append == 1 means we are appending instead of overwriting.
  * append == 2 means we are prepending instead of overwriting.
  *
- * nonamechange means don't change the current filename, it is ignored
- * if tmp is nonzero or if we're appending/prepending.
+ * nonamechange means don't change the current filename.  It is ignored
+ * if tmp is FALSE or if we're appending/prepending.
  *
  * Return -1 on error, 1 on success. */
 int write_file(const char *name, int tmp, int append, int nonamechange)
@@ -1455,7 +1453,7 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
     /* If NOFOLLOW_SYMLINKS is set, it doesn't make sense to prepend or
      * append to a symlink.  Here we warn about the contradiction. */
     if (ISSET(NOFOLLOW_SYMLINKS) && anyexists && S_ISLNK(lst.st_mode)) {
-	statusbar(_("Cannot prepend or append to a symlink with --nofollow set."));
+	statusbar(_("Cannot prepend or append to a symlink with --nofollow set"));
 	goto cleanup_and_exit;
     }
 
@@ -1640,7 +1638,7 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 	goto cleanup_and_exit;
     }
 
-    /* There might not be a magic line.  There won't be when writing out
+    /* There might not be a magicline.  There won't be when writing out
      * a selection. */
     assert(fileage != NULL && filebot != NULL);
     while (fileptr != filebot) {
@@ -1739,11 +1737,11 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 #ifndef NANO_SMALL
 /* Write a marked selection from a file out.  First, set fileage and
  * filebot as the top and bottom of the mark, respectively.  Then call
- * write_file() with the values of name, temp, append, and nonamechange.
- * Finally, set fileage and filebot back to their old values and
- * return. */
-int write_marked(const char *name, int tmp, int append, int
-	nonamechange)
+ * write_file() with the values of name, temp, and append, and with
+ * nonamechange set to TRUE so that we don't change the current
+ * filename.  Finally, set fileage and filebot back to their old values
+ * and return. */
+int write_marked(const char *name, int tmp, int append)
 {
     int retval = -1;
     filestruct *fileagebak = fileage;
@@ -1785,7 +1783,7 @@ int write_marked(const char *name, int tmp, int append, int
     if (filebot->data[0] != '\0' && filebot->next != NULL)
 	filebot = filebot->next;
 
-    retval = write_file(name, tmp, append, nonamechange);
+    retval = write_file(name, tmp, append, TRUE);
 
     /* Now restore everything. */
     fileage->data -= topx;
@@ -1811,24 +1809,12 @@ int do_writeout(int exiting)
     currshortcut = writefile_list;
 #endif
 
-    if (exiting && ISSET(TEMP_OPT)) {
-	i = -1;
-	if (filename[0] != '\0') {
-	    i = write_file(filename, FALSE, 0, FALSE);
-	    if (i == 1) {
-		/* Write succeeded. */
-		display_main_list();
-		return 1;
-	    }
-	}
-
-	/* No filename or the write above failed. */
-	if (i == -1) {
-	    UNSET(TEMP_OPT);
-	    do_exit();
-
-	    /* They cancelled; abort quit. */
-	    return -1;
+    if (exiting && filename[0] != '\0' && ISSET(TEMP_OPT)) {
+	i = write_file(filename, FALSE, 0, FALSE);
+	if (i == 1) {
+	    /* Write succeeded. */
+	    display_main_list();
+	    return 1;
 	}
     }
 
@@ -1941,25 +1927,32 @@ int do_writeout(int exiting)
 	    struct stat st;
 
 	    if (!stat(answer, &st)) {
-		i = do_yesno(FALSE, _("File exists, OVERWRITE ?"));
+		i = do_yesno(FALSE, _("File exists, OVERWRITE ? "));
 		if (i == 0 || i == -1)
 		    continue;
-	    } else if (filename[0] != '\0'
+	    /* If we're using restricted mode, we aren't allowed to
+	     * change the name of a file once it has one because that
+	     * would allow reading from or writing to files not
+	     * specified on the command line.  In this case, don't
+	     * bother showing the "Different Name" prompt. */
+	    } else if (!ISSET(RESTRICTED) && filename[0] != '\0'
 #ifndef NANO_SMALL
-		&& (!ISSET(MARK_ISSET) || exiting)
+		&& (exiting || !ISSET(MARK_ISSET))
 #endif
 		) {
-		i = do_yesno(FALSE, _("Save file under DIFFERENT NAME ?"));
+		i = do_yesno(FALSE, _("Save file under DIFFERENT NAME ?" ));
 		if (i == 0 || i == -1)
 		    continue;
 	    }
 	}
 
 #ifndef NANO_SMALL
-	/* Here's where we allow the selected text to be written to
-	 * a separate file. */
-	if (ISSET(MARK_ISSET) && !exiting)
-	    i = write_marked(answer, FALSE, append, FALSE);
+	/* Here's where we allow the selected text to be written to a
+	 * separate file.  If we're using restricted mode, this is
+	 * disabled since it allows reading from or writing to files not
+	 * specified on the command line. */
+	if (!ISSET(RESTRICTED) && !exiting && ISSET(MARK_ISSET))
+	    i = write_marked(answer, FALSE, append);
 	else
 #endif /* !NANO_SMALL */
 	    i = write_file(answer, FALSE, append, FALSE);
@@ -2245,7 +2238,7 @@ char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
 	tmp = matchbuf;
 
 	/* skip any leading white space */
-	while (*tmp && isspace((int)*tmp))
+	while (*tmp && isblank(*tmp))
 	    ++tmp;
 
 	/* Free up any memory already allocated */
@@ -2435,7 +2428,6 @@ char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
 }
 #endif /* !DISABLE_TABCOMP */
 
-#if !defined(DISABLE_BROWSER) || !defined(NANO_SMALL)
 /* Only print the last part of a path; isn't there a shell
  * command for this? */
 const char *tail(const char *foo)
@@ -2450,7 +2442,6 @@ const char *tail(const char *foo)
 
     return tmp;
 }
-#endif
 
 #ifndef DISABLE_BROWSER
 /* Our sort routine for file listings -- sort directories before
@@ -2471,7 +2462,7 @@ int diralphasort(const void *va, const void *vb)
 }
 
 /* Free our malloc()ed memory */
-void free_charptrarray(char **array, int len)
+void free_charptrarray(char **array, size_t len)
 {
     for (; len > 0; len--)
 	free(array[len - 1]);
@@ -2608,7 +2599,7 @@ char *do_browser(const char *inpath)
 	char *new_path;
 	    /* Used by the Go To Directory prompt. */
 
-	blank_statusbar_refresh();
+	check_statblank();
 
 #if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
 	currshortcut = browser_list;
@@ -2677,7 +2668,7 @@ char *do_browser(const char *inpath)
 	case NANO_PREVPAGE_FKEY:
 	case '-': /* Pico compatibility */
 	    if (selected >= (editwinrows + lineno % editwinrows) * width)
-		selected -= (editwinrows + lineno % editwinrows) * width; 
+		selected -= (editwinrows + lineno % editwinrows) * width;
 	    else
 		selected = 0;
 	    break;
@@ -2691,8 +2682,9 @@ char *do_browser(const char *inpath)
 	case NANO_HELP_KEY:
 	case NANO_HELP_FKEY:
 	case '?': /* Pico compatibility */
-	     do_help();
-	     break;
+	    do_help();
+	    curs_set(0);
+	    break;
 	case NANO_ENTER_KEY:
 	case 'S': /* Pico compatibility */
 	case 's':
@@ -2967,7 +2959,7 @@ void load_history(void)
             if (errno != ENOENT) {
 		/* Don't save history when we quit. */
 		UNSET(HISTORYLOG);
-		rcfile_error(_("Unable to open ~/.nano_history file, %s"), strerror(errno));
+		rcfile_error(_("Unable to open ~/.nano_history file: %s"), strerror(errno));
 	    }
 	    free(nanohist);
 	} else {
@@ -3017,28 +3009,28 @@ void save_history(void)
     if (homenv != NULL || userage != NULL) {
 	hist = fopen(nanohist, "wb");
 	if (hist == NULL) {
-	    rcfile_msg(_("Unable to write ~/.nano_history file, %s"), strerror(errno));
+	    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
 	} else {
 	    /* set rw only by owner for security ?? */
 	    chmod(nanohist, S_IRUSR | S_IWUSR);
 	    /* write oldest first */
-	    for (h = search_history.tail ; h->prev ; h = h->prev) {
+	    for (h = search_history.tail; h->prev; h = h->prev) {
 		h->data = charealloc(h->data, strlen(h->data) + 2);
 		strcat(h->data, "\n");
 		if (fputs(h->data, hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file, %s"), strerror(errno));
+		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
 		    goto come_from;
 		}
 	    }
 	    if (fputs("\n", hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file, %s"), strerror(errno));
+		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
 		    goto come_from;
 	    }
-	    for (h = replace_history.tail ; h->prev ; h = h->prev) {
+	    for (h = replace_history.tail; h->prev; h = h->prev) {
 		h->data = charealloc(h->data, strlen(h->data) + 2);
 		strcat(h->data, "\n");
 		if (fputs(h->data, hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file, %s"), strerror(errno));
+		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
 		    goto come_from;
 		}
 	    }
