@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.19 2004/03/19 21:46:34 dolorous Exp $ */
+/* $Id: global.c,v 1.32 2004/05/29 20:38:08 dolorous Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -66,7 +66,18 @@ filestruct *cutbuffer = NULL;	/* A place to store cut text */
 openfilestruct *open_files = NULL;	/* The list of open files */
 #endif
 
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+char *whitespace = NULL;	/* Characters used when displaying
+				   the first characters of tabs and
+				   spaces. */
+#endif
+
 #ifndef DISABLE_JUSTIFY
+char *punct = NULL;		/* Closing punctuation that can end
+				   sentences. */
+char *brackets = NULL;		/* Closing brackets that can follow
+				   closing punctuation and can end
+				   sentences. */
 char *quotestr = NULL;		/* Quote string.  The default value is
 				   set in main(). */
 #endif
@@ -231,18 +242,21 @@ void toggle_init(void)
     char *toggle_const_msg, *toggle_autoindent_msg, *toggle_suspend_msg,
 	*toggle_nohelp_msg, *toggle_cuttoend_msg,
 	*toggle_noconvert_msg, *toggle_dos_msg, *toggle_mac_msg,
-	*toggle_backup_msg, *toggle_smooth_msg;
+	*toggle_backup_msg, *toggle_smooth_msg, *toggle_smarthome_msg;
 #ifndef DISABLE_MOUSE
     char *toggle_mouse_msg;
 #endif
 #ifndef DISABLE_WRAPPING
     char *toggle_wrap_msg;
 #endif
+#ifdef ENABLE_COLOR
+    char *toggle_syntax_msg;
+#endif
 #ifdef ENABLE_MULTIBUFFER
     char *toggle_multibuffer_msg;
 #endif
-#ifdef ENABLE_COLOR
-    char *toggle_syntax_msg;
+#ifdef ENABLE_NANORC
+    char *toggle_whitespace_msg;
 #endif
 
     /* There is no need to reinitialize the toggles.  They can't
@@ -263,19 +277,26 @@ void toggle_init(void)
     toggle_mac_msg = _("Writing file in Mac format");
     toggle_backup_msg = _("Backing up file");
     toggle_smooth_msg = _("Smooth scrolling");
-#ifdef ENABLE_COLOR
-    toggle_syntax_msg = _("Color syntax highlighting");
-#endif
+    toggle_smarthome_msg = _("Smart home key");
 #ifndef DISABLE_WRAPPING
     toggle_wrap_msg = _("Auto line wrap");
 #endif
+#ifdef ENABLE_COLOR
+    toggle_syntax_msg = _("Color syntax highlighting");
+#endif
 #ifdef ENABLE_MULTIBUFFER
     toggle_multibuffer_msg = _("Multiple file buffers");
 #endif
+#ifdef ENABLE_NANORC
+    toggle_whitespace_msg = _("Whitespace display");
+#endif
 
     toggle_init_one(TOGGLE_NOHELP_KEY, toggle_nohelp_msg, NO_HELP);
 #ifdef ENABLE_MULTIBUFFER
-    toggle_init_one(TOGGLE_MULTIBUFFER_KEY, toggle_multibuffer_msg, MULTIBUFFER);
+    /* If we're using restricted mode, the multibuffer toggle is
+     * disabled.  It's useless since inserting files is disabled. */
+    if (!ISSET(RESTRICTED))
+	toggle_init_one(TOGGLE_MULTIBUFFER_KEY, toggle_multibuffer_msg, MULTIBUFFER);
 #endif
     toggle_init_one(TOGGLE_CONST_KEY, toggle_const_msg, CONSTUPDATE);
     toggle_init_one(TOGGLE_AUTOINDENT_KEY, toggle_autoindent_msg, AUTOINDENT);
@@ -283,18 +304,31 @@ void toggle_init(void)
     toggle_init_one(TOGGLE_WRAP_KEY, toggle_wrap_msg, NO_WRAP);
 #endif
     toggle_init_one(TOGGLE_CUTTOEND_KEY, toggle_cuttoend_msg, CUT_TO_END);
-    toggle_init_one(TOGGLE_SUSPEND_KEY, toggle_suspend_msg, SUSPEND);
+    /* If we're using restricted mode, the suspend toggle is disabled.
+     * It's useless since suspending is disabled. */
+    if (!ISSET(RESTRICTED))
+	toggle_init_one(TOGGLE_SUSPEND_KEY, toggle_suspend_msg, SUSPEND);
 #ifndef DISABLE_MOUSE
     toggle_init_one(TOGGLE_MOUSE_KEY, toggle_mouse_msg, USE_MOUSE);
 #endif
-    toggle_init_one(TOGGLE_NOCONVERT_KEY, toggle_noconvert_msg, NO_CONVERT);
-    toggle_init_one(TOGGLE_DOS_KEY, toggle_dos_msg, DOS_FILE);
-    toggle_init_one(TOGGLE_MAC_KEY, toggle_mac_msg, MAC_FILE);
-    toggle_init_one(TOGGLE_BACKUP_KEY, toggle_backup_msg, BACKUP_FILE);
+    /* If we're using restricted mode, the no-conversion, DOS format,
+     * Mac format, and backup toggles are disabled.  The first, second,
+     * and third are useless since inserting files is disabled, and the
+     * fourth is useless since backups are disabled. */
+    if (!ISSET(RESTRICTED)) {
+	toggle_init_one(TOGGLE_NOCONVERT_KEY, toggle_noconvert_msg, NO_CONVERT);
+	toggle_init_one(TOGGLE_DOS_KEY, toggle_dos_msg, DOS_FILE);
+	toggle_init_one(TOGGLE_MAC_KEY, toggle_mac_msg, MAC_FILE);
+	toggle_init_one(TOGGLE_BACKUP_KEY, toggle_backup_msg, BACKUP_FILE);
+    }
     toggle_init_one(TOGGLE_SMOOTH_KEY, toggle_smooth_msg, SMOOTHSCROLL);
+    toggle_init_one(TOGGLE_SMARTHOME_KEY, toggle_smarthome_msg, SMART_HOME);
 #ifdef ENABLE_COLOR
     toggle_init_one(TOGGLE_SYNTAX_KEY, toggle_syntax_msg, COLOR_SYNTAX);
 #endif
+#ifdef ENABLE_NANORC
+    toggle_init_one(TOGGLE_WHITESPACE_KEY, toggle_whitespace_msg, WHITESPACE_DISPLAY);
+#endif
 }
 
 #ifdef DEBUG
@@ -348,7 +382,7 @@ void shortcut_init(int unjustify)
 	"", *nano_execute_msg = "", *nano_dos_msg =
 	"", *nano_mac_msg = "", *nano_backup_msg =
 	"", *nano_editstr_msg = "", *nano_parabegin_msg =
-	"", *nano_paraend_msg = "";
+	"", *nano_paraend_msg = "", *nano_fulljustify_msg = "";
 
 #ifdef ENABLE_MULTIBUFFER
     const char *nano_openprev_msg = "", *nano_opennext_msg =
@@ -412,6 +446,7 @@ void shortcut_init(int unjustify)
     nano_editstr_msg = _("Edit the previous search/replace strings");
     nano_parabegin_msg = _("Go to the beginning of the current paragraph");
     nano_paraend_msg = _("Go to the end of the current paragraph");
+    nano_fulljustify_msg = _("Justify the entire file");
 #ifdef HAVE_REGEX_H
     nano_regexp_msg = _("Use regular expressions");
     nano_bracket_msg = _("Find other bracket");
@@ -425,9 +460,9 @@ void shortcut_init(int unjustify)
 
     free_shortcutage(&main_list);
 
-/* The following macro is to be used in calling sc_init_one.  The point is
- * that sc_init_one takes 9 arguments, unless DISABLE_HELP is defined,
- * when the fourth one should not be there. */
+/* The following macro is to be used in calling sc_init_one().  The
+ * point is that sc_init_one() takes 9 arguments, unless DISABLE_HELP is
+ * defined, when the 4th one should not be there. */
 #ifndef DISABLE_HELP
 #  define IFHELP(help, nextvar) help, nextvar
 #else
@@ -437,7 +472,13 @@ void shortcut_init(int unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
 #ifdef ENABLE_MULTIBUFFER
     if (open_files != NULL && (open_files->prev != NULL || open_files->next != NULL))
@@ -461,9 +502,19 @@ void shortcut_init(int unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_JUSTIFY_KEY, _("Justify"),
 		IFHELP(nano_justify_msg, NANO_NO_KEY),
-		NANO_JUSTIFY_FKEY, NANO_NO_KEY, NOVIEW, do_justify);
+		NANO_JUSTIFY_FKEY, NANO_NO_KEY, NOVIEW,
+#ifndef NANO_SMALL
+		do_justify_void
+#else
+		nano_disabled_msg
+#endif
+		);
 
-    /* this is so we can view multiple files */
+    /* We allow inserting files in view mode if multibuffers are
+     * available, so that we can view multiple files. */
+    /* If we're using restricted mode, inserting files is disabled since
+     * it allows reading from or writing to files not specified on the
+     * command line. */
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_INSERTFILE_KEY, _("Read File"),
 		IFHELP(nano_insert_msg, NANO_NO_KEY), NANO_INSERTFILE_FKEY,
@@ -473,7 +524,7 @@ void shortcut_init(int unjustify)
 #else
 		NOVIEW
 #endif
-		, do_insertfile_void);
+		, !ISSET(RESTRICTED) ? do_insertfile_void : nano_disabled_msg);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_WHEREIS_KEY, _("Where Is"),
@@ -511,10 +562,17 @@ void shortcut_init(int unjustify)
 		IFHELP(nano_cursorpos_msg, NANO_NO_KEY), NANO_CURSORPOS_FKEY,
 		NANO_NO_KEY, VIEW, do_cursorpos_void);
 
+    /* If we're using restricted mode, spell checking is disabled
+     * because it allows reading from or writing to files not specified
+     * on the command line. */
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_SPELL_KEY, _("To Spell"),
 		IFHELP(nano_spell_msg, NANO_NO_KEY), NANO_SPELL_FKEY,
-		NANO_NO_KEY, NOVIEW, do_spell);
+		NANO_NO_KEY, NOVIEW,
+#ifndef DISABLE_SPELLER
+		!ISSET(RESTRICTED) ? do_spell :
+#endif
+		nano_disabled_msg);
 
     sc_init_one(&main_list, NANO_GOTO_KEY, _("Go To Line"),
 		IFHELP(nano_goto_msg, NANO_ALT_GOTO_KEY), NANO_GOTO_FKEY,
@@ -534,11 +592,11 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&main_list, NANO_FORWARD_KEY, _("Forward"),
 		IFHELP(nano_forward_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, VIEW, do_right);
+		NANO_NO_KEY, VIEW, do_right_void);
 
     sc_init_one(&main_list, NANO_BACK_KEY, _("Back"),
 		IFHELP(nano_back_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, VIEW, do_left);
+		NANO_NO_KEY, VIEW, do_left_void);
 
     sc_init_one(&main_list, NANO_HOME_KEY, _("Home"),
 		IFHELP(nano_home_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -554,7 +612,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&main_list, NANO_MARK_KEY, _("Mark Text"),
 		IFHELP(nano_mark_msg, NANO_ALT_MARK_KEY),
-		NANO_NO_KEY, NANO_NO_KEY, NOVIEW, do_mark);
+		NANO_NO_KEY, NANO_NO_KEY, NOVIEW,
+#ifndef NANO_SMALL
+		do_mark
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&main_list, NANO_DELETE_KEY, _("Delete"),
 		IFHELP(nano_delete_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -596,7 +660,8 @@ void shortcut_init(int unjustify)
 		NANO_OPENNEXT_ALTKEY, VIEW, open_nextfile_void);
 #endif
 
-#if !defined(NANO_SMALL) && defined(HAVE_REGEX_H)
+#ifndef NANO_SMALL
+#ifdef HAVE_REGEX_H
     sc_init_one(&main_list, NANO_NO_KEY, _("Find Other Bracket"),
 		IFHELP(nano_bracket_msg, NANO_BRACKET_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, VIEW, do_find_bracket);
@@ -605,12 +670,19 @@ void shortcut_init(int unjustify)
     sc_init_one(&main_list, NANO_NO_KEY, _("Where Is Next"),
 		IFHELP(nano_whereis_next_msg, NANO_WHEREIS_NEXT_KEY),
 		NANO_NO_KEY, NANO_NO_KEY, VIEW, do_research);
+#endif
 
     free_shortcutage(&whereis_list);
 
     sc_init_one(&whereis_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&whereis_list, NANO_CANCEL_KEY, _("Cancel"),
@@ -647,6 +719,11 @@ void shortcut_init(int unjustify)
     sc_init_one(&whereis_list, NANO_PARAEND_KEY, _("End of Par"),
 		IFHELP(nano_paraend_msg, NANO_NO_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, VIEW, do_para_end);
+
+    /* Translators: try to keep this string under 10 characters long */
+    sc_init_one(&whereis_list, NANO_FULLJUSTIFY_KEY, _("FullJstify"),
+		IFHELP(nano_fulljustify_msg, NANO_NO_KEY), NANO_NO_KEY,
+		NANO_NO_KEY, NOVIEW, do_full_justify);
 #endif
 
 #ifndef NANO_SMALL
@@ -680,7 +757,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&replace_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&replace_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -727,7 +810,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&replace_list_2, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&replace_list_2, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -751,7 +840,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&goto_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&goto_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -770,55 +865,83 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&help_list, NANO_PREVPAGE_KEY, _("Prev Page"),
 		IFHELP(nano_prevpage_msg, NANO_NO_KEY), NANO_PREVPAGE_FKEY,
-		NANO_NO_KEY, VIEW, do_page_up);
+		NANO_NO_KEY, VIEW, 0);
 
     sc_init_one(&help_list, NANO_NEXTPAGE_KEY, _("Next Page"),
 		IFHELP(nano_nextpage_msg, NANO_NO_KEY), NANO_NEXTPAGE_FKEY,
-		NANO_NO_KEY, VIEW, do_page_down);
+		NANO_NO_KEY, VIEW, 0);
+
+    sc_init_one(&help_list, NANO_PREVLINE_KEY, _("Prev Line"),
+		IFHELP(nano_prevline_msg, NANO_NO_KEY), NANO_NO_KEY,
+		NANO_NO_KEY, VIEW, 0);
+
+    sc_init_one(&help_list, NANO_NEXTLINE_KEY, _("Next Line"),
+		IFHELP(nano_nextline_msg, NANO_NO_KEY), NANO_NO_KEY,
+		NANO_NO_KEY, VIEW, 0);
 
     sc_init_one(&help_list, NANO_EXIT_KEY, _("Exit"),
 		IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-		NANO_NO_KEY, VIEW, do_exit);
+		NANO_NO_KEY, VIEW, 0);
 #endif
 
     free_shortcutage(&writefile_list);
 
     sc_init_one(&writefile_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
 #ifndef DISABLE_BROWSER
+    /* If we're using restricted mode, the file browser is disabled.
+     * It's useless since inserting files is disabled. */
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_TOFILES_KEY, _("To Files"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_TOFILES_KEY, _("To Files"),
 		IFHELP(nano_tofiles_msg, NANO_NO_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 #endif
 
 #ifndef NANO_SMALL
+    /* If we're using restricted mode, the DOS format, Mac format,
+     * append, prepend, and backup toggles are disabled.  The first and
+     * second are useless since inserting files is disabled, the third
+     * and fourth are disabled because they allow writing to files not
+     * specified on the command line, and the fifth is useless since
+     * backups are disabled. */
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_NO_KEY, _("DOS Format"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_NO_KEY, _("DOS Format"),
 		IFHELP(nano_dos_msg, TOGGLE_DOS_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_NO_KEY, _("Mac Format"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_NO_KEY, _("Mac Format"),
 		IFHELP(nano_mac_msg, TOGGLE_MAC_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 #endif
 
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_NO_KEY, _("Append"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_NO_KEY, _("Append"),
 		IFHELP(nano_append_msg, NANO_APPEND_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_NO_KEY, _("Prepend"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_NO_KEY, _("Prepend"),
 		IFHELP(nano_prepend_msg, NANO_PREPEND_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 
 #ifndef NANO_SMALL
     /* Translators: try to keep this string under 16 characters long */
-    sc_init_one(&writefile_list, NANO_NO_KEY, _("Backup File"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&writefile_list, NANO_NO_KEY, _("Backup File"),
 		IFHELP(nano_backup_msg, TOGGLE_BACKUP_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 #endif
@@ -831,27 +954,42 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&insertfile_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&insertfile_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, VIEW, 0);
 
 #ifndef DISABLE_BROWSER
-    sc_init_one(&insertfile_list, NANO_TOFILES_KEY, _("To Files"),
+    /* If we're using restricted mode, the file browser is disabled.
+     * It's useless since inserting files is disabled. */
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&insertfile_list, NANO_TOFILES_KEY, _("To Files"),
 		IFHELP(nano_tofiles_msg, NANO_NO_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 #endif
 
 #ifndef NANO_SMALL
+    /* If we're using restricted mode, command execution is disabled.
+     * It's useless since inserting files is disabled. */
     /* Translators: try to keep this string under 22 characters long */
-    sc_init_one(&insertfile_list, NANO_EXTCMD_KEY, _("Execute Command"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&insertfile_list, NANO_EXTCMD_KEY, _("Execute Command"),
 		IFHELP(nano_execute_msg, NANO_NO_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 
 #ifdef ENABLE_MULTIBUFFER
+    /* If we're using restricted mode, the multibuffer toggle is
+     * disabled.  It's useless since inserting files is disabled. */
     /* Translators: try to keep this string under 22 characters long */
-    sc_init_one(&insertfile_list, NANO_NO_KEY, _("New Buffer"),
+    if (!ISSET(RESTRICTED))
+	sc_init_one(&insertfile_list, NANO_NO_KEY, _("New Buffer"),
 		IFHELP(nano_multibuffer_msg, TOGGLE_MULTIBUFFER_KEY), NANO_NO_KEY,
 		NANO_NO_KEY, NOVIEW, 0);
 #endif
@@ -862,7 +1000,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&spell_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&spell_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -874,7 +1018,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&extcmd_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&extcmd_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -886,7 +1036,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&browser_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&browser_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -909,7 +1065,13 @@ void shortcut_init(int unjustify)
 
     sc_init_one(&gotodir_list, NANO_HELP_KEY, _("Get Help"),
 		IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-		NANO_NO_KEY, VIEW, do_help);
+		NANO_NO_KEY, VIEW,
+#ifndef DISABLE_HELP
+		do_help
+#else
+		nano_disabled_msg
+#endif
+		);
 
     sc_init_one(&gotodir_list, NANO_CANCEL_KEY, _("Cancel"),
 		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
