@@ -1,4 +1,4 @@
-/* $Id: move.c,v 1.5 2004/01/09 23:04:55 dolorous Exp $ */
+/* $Id: move.c,v 1.9 2004/05/30 03:19:52 dolorous Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
@@ -23,53 +23,99 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <assert.h>
 #include "proto.h"
 #include "nano.h"
 
+int do_first_line(void)
+{
+    int old_pww = placewewant;
+    current = fileage;
+    placewewant = 0;
+    current_x = 0;
+    if (edittop != fileage || need_vertical_update(old_pww))
+	edit_update(current, TOP);
+    return 1;
+}
+
+int do_last_line(void)
+{
+    int old_pww = placewewant;
+    current = filebot;
+    placewewant = 0;
+    current_x = 0;
+    if (edittop->lineno + (editwinrows / 2) != filebot->lineno ||
+	need_vertical_update(old_pww))
+	edit_update(current, CENTER);
+    return 1;
+}
+
 int do_home(void)
 {
-    current_x = 0;
-    placewewant = 0;
+    int old_pww = placewewant;
+#ifndef NANO_SMALL
+    if (ISSET(SMART_HOME)) {
+	int old_current_x = current_x;
+
+	for (current_x = 0; isblank(current->data[current_x]) &&
+		current->data[current_x] != '\0'; current_x++)
+	    ;
+
+	if (current_x == old_current_x || current->data[current_x] == '\0')
+	    current_x = 0;
+
+	placewewant = xplustabs();
+    } else {
+#endif
+	current_x = 0;
+	placewewant = 0;
+#ifndef NANO_SMALL
+    }
+#endif
     check_statblank();
-    update_line(current, current_x);
+    if (need_horizontal_update(old_pww))
+	update_line(current, current_x);
     return 1;
 }
 
 int do_end(void)
 {
+    int old_pww = placewewant;
     current_x = strlen(current->data);
     placewewant = xplustabs();
     check_statblank();
-    update_line(current, current_x);
+    if (need_horizontal_update(old_pww))
+	update_line(current, current_x);
     return 1;
 }
 
 int do_page_up(void)
 {
-    int i;
-
+    int old_pww = placewewant;
+    const filestruct *old_current = current;
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
 
-    /* If edittop is the first line of the file, move current up there
+    /* If the first line of the file is onscreen, move current up there
      * and put the cursor at the beginning of the line. */
     if (edittop == fileage) {
 	current = fileage;
 	placewewant = 0;
     } else {
-	/* Move the top line of the edit window up a page. */
-	for (i = 0; i < editwinrows - 2 && edittop->prev != NULL; i++)
-	    edittop = edittop->prev;
+	edit_scroll(UP, editwinrows - 2);
+
 #ifndef NANO_SMALL
-	/* If we're in smooth scrolling mode and there was at least one
+	/* If we're in smooth scrolling mode and there's at least one
 	 * page of text left, move the current line of the edit window
 	 * up a page. */
-	if (ISSET(SMOOTHSCROLL) && current->lineno > editwinrows - 2)
+	if (ISSET(SMOOTHSCROLL) && current->lineno > editwinrows - 2) {
+	    int i;
 	    for (i = 0; i < editwinrows - 2; i++)
 		current = current->prev;
-	/* If we're not in smooth scrolling mode and there was at least
+	}
+	/* If we're not in smooth scrolling mode or there isn't at least
 	 * one page of text left, put the cursor at the beginning of the
 	 * top line of the edit window, as Pico does. */
 	else {
@@ -80,10 +126,12 @@ int do_page_up(void)
 	}
 #endif
     }
+
     /* Get the equivalent x-coordinate of the new line. */
     current_x = actual_x(current->data, placewewant);
 
-    edit_refresh();
+    /* Update all the lines that need to be updated. */
+    edit_redraw(old_current, old_pww);
 
     check_statblank();
     return 1;
@@ -91,8 +139,8 @@ int do_page_up(void)
 
 int do_page_down(void)
 {
-    int i;
-
+    int old_pww = placewewant;
+    const filestruct *old_current = current;
 #ifndef DISABLE_WRAPPING
     wrap_reset();
 #endif
@@ -103,17 +151,19 @@ int do_page_down(void)
 	current = filebot;
 	placewewant = 0;
     } else {
-	/* Move the top line of the edit window down a page. */
-	for (i = 0; i < editwinrows - 2; i++)
-	    edittop = edittop->next;
+	edit_scroll(DOWN, editwinrows - 2);
+
 #ifndef NANO_SMALL
-	/* If we're in smooth scrolling mode and there was at least one
+	/* If we're in smooth scrolling mode and there's at least one
 	 * page of text left, move the current line of the edit window
 	 * down a page. */
-	if (ISSET(SMOOTHSCROLL) && current->lineno + editwinrows - 2 <= filebot->lineno)
+	if (ISSET(SMOOTHSCROLL) && current->lineno + editwinrows - 2 <=
+		filebot->lineno) {
+	    int i;
 	    for (i = 0; i < editwinrows - 2; i++)
 		current = current->next;
-	/* If we're not in smooth scrolling mode and there was at least
+	}
+	/* If we're not in smooth scrolling mode or there isn't at least
 	 * one page of text left, put the cursor at the beginning of the
 	 * top line of the edit window, as Pico does. */
 	else {
@@ -124,10 +174,12 @@ int do_page_down(void)
 	}
 #endif
     }
+
     /* Get the equivalent x-coordinate of the new line. */
     current_x = actual_x(current->data, placewewant);
 
-    edit_refresh();
+    /* Update all the lines that need to be updated. */
+    edit_redraw(old_current, old_pww);
 
     check_statblank();
     return 1;
@@ -146,18 +198,25 @@ int do_up(void)
     assert(current_y == current->lineno - edittop->lineno);
     current = current->prev;
     current_x = actual_x(current->data, placewewant);
-    if (current_y > 0) {
-	update_line(current->next, 0);
-	    /* It was necessary to change current first, so that the
-	     * mark display will change! */
-	update_line(current, current_x);
-    } else
+
+    /* If we're on the first row of the edit window, scroll up one line
+     * if we're in smooth scrolling mode, or up half a page if we're
+     * not. */
+    if (current_y == 0)
+	edit_scroll(UP,
 #ifndef NANO_SMALL
-    if (ISSET(SMOOTHSCROLL))
-	edit_update(current, TOP);
-    else
+		ISSET(SMOOTHSCROLL) ? 1 :
 #endif
-	edit_update(current, CENTER);
+		editwinrows / 2);
+
+    /* Update the lines left alone by edit_scroll(): the line we were on
+     * before and the line we're on now.  The former needs to be redrawn
+     * if we're not on the first page, and the latter needs to be
+     * drawn. */
+    if (need_vertical_update(0))
+	update_line(current->next, 0);
+    update_line(current, current_x);
+
     return 1;
 }
 
@@ -177,25 +236,30 @@ int do_down(void)
     current = current->next;
     current_x = actual_x(current->data, placewewant);
 
-    /* Note that current_y is zero-based.  This test checks for the
-     * cursor's being not on the last row of the edit window. */
-    if (current_y != editwinrows - 1) {
-	update_line(current->prev, 0);
-	update_line(current, current_x);
-    } else
+    /* If we're on the last row of the edit window, scroll down one line
+     * if we're in smooth scrolling mode, or down half a page if we're
+     * not. */
+    if (current_y == editwinrows - 1)
+	edit_scroll(DOWN,
 #ifndef NANO_SMALL
-    if (ISSET(SMOOTHSCROLL))
-	/* In this case current_y does not change.  The cursor remains
-	 * at the bottom of the edit window. */
-	edit_update(edittop->next, TOP);
-    else
+		ISSET(SMOOTHSCROLL) ? 1 :
 #endif
-	edit_update(current, CENTER);
+		editwinrows / 2);
+
+    /* Update the lines left alone by edit_scroll(): the line we were on
+     * before and the line we're on now.  The former needs to be redrawn
+     * if we're not on the first page, and the latter needs to be
+     * drawn. */
+    if (need_vertical_update(0))
+	update_line(current->prev, 0);
+    update_line(current, current_x);
+
     return 1;
 }
 
-int do_left(void)
+int do_left(int allow_update)
 {
+    int old_pww = placewewant;
     if (current_x > 0)
 	current_x--;
     else if (current != fileage) {
@@ -204,12 +268,19 @@ int do_left(void)
     }
     placewewant = xplustabs();
     check_statblank();
-    update_line(current, current_x);
+    if (allow_update && need_horizontal_update(old_pww))
+	update_line(current, current_x);
     return 1;
 }
 
-int do_right(void)
+int do_left_void(void)
 {
+    return do_left(TRUE);
+}
+
+int do_right(int allow_update)
+{
+    int old_pww = placewewant;
     assert(current_x <= strlen(current->data));
 
     if (current->data[current_x] != '\0')
@@ -220,6 +291,12 @@ int do_right(void)
     }
     placewewant = xplustabs();
     check_statblank();
-    update_line(current, current_x);
+    if (allow_update && need_horizontal_update(old_pww))
+	update_line(current, current_x);
     return 1;
 }
+
+int do_right_void(void)
+{
+    return do_right(TRUE);
+}
