@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.26 2004/03/30 04:17:10 dolorous Exp $ */
+/* $Id: nano.h,v 1.41 2004/05/29 16:25:30 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -54,9 +54,10 @@
 #endif
 
 #ifdef USE_SLANG
-/* Slang support enabled.  Work around Slang's not defining KEY_DC or
- * KEY_IC. */
+/* Slang support enabled. */
 #include <slcurses.h>
+/* Slang curses emulation brain damage, part 2: Slang doesn't define the
+ * curses equivalents of the Insert or Delete keys. */
 #define KEY_DC SL_KEY_DELETE
 #define KEY_IC SL_KEY_IC
 #elif defined(HAVE_NCURSES_H)
@@ -91,7 +92,12 @@
 # endif
 #endif
 
-/* If no strcasecmp()/strncasecmp(), use the versions we have. */
+/* If no isblank(), strcasecmp(), strncasecmp(), strcasestr(), or
+ * strnlen(), use the versions we have. */
+#ifndef HAVE_ISBLANK
+#define isblank is_blank_char
+#endif
+
 #ifndef HAVE_STRCASECMP
 #define strcasecmp nstricmp
 #endif
@@ -100,39 +106,19 @@
 #define strncasecmp nstrnicmp
 #endif
 
+#ifndef HAVE_STRCASESTR
+#define strcasestr nstristr
+#endif
+
+#ifndef HAVE_STRNLEN
+#define strnlen nstrnlen
+#endif
+
 /* Assume ERR is defined as -1.  To avoid duplicate case values when
  * some key definitions are missing, we have to set all of these, and
  * all of the special sentinel values below, to different negative
  * values other than -1. */
 
-/* HP-UX 10 & 11 do not seem to support KEY_HOME and KEY_END. */
-#ifndef KEY_HOME
-#define KEY_HOME -2
-#endif
-
-#ifndef KEY_END
-#define KEY_END -3
-#endif
-
-/* Slang and SunOS 5.7-5.9 do not seem to support KEY_RESIZE. */
-#ifndef KEY_RESIZE
-#define KEY_RESIZE -4
-#endif
-
-/* Slang does not seem to support KEY_SUSPEND, KEY_SLEFT, or
- * KEY_SRIGHT. */
-#ifndef KEY_SUSPEND
-#define KEY_SUSPEND -5
-#endif
-
-#ifndef KEY_SLEFT
-#define KEY_SLEFT -6
-#endif
-
-#ifndef KEY_SRIGHT
-#define KEY_SRIGHT -7
-#endif
-
 #define VERMSG "GNU nano " VERSION
 
 /* If we aren't using ncurses, turn the mouse support off, as it's
@@ -267,35 +253,36 @@ typedef struct historyheadtype {
 /* Bitwise flags so we can save space (or more correctly, not waste
  * it). */
 #define MODIFIED		(1<<0)
-#define KEEP_CUTBUFFER		(1<<1)
-#define CASE_SENSITIVE		(1<<2)
-#define MARK_ISSET		(1<<3)
-#define CONSTUPDATE		(1<<4)
-#define NO_HELP			(1<<5)
-#define NOFOLLOW_SYMLINKS	(1<<6)
-#define SUSPEND			(1<<7)
-#define NO_WRAP			(1<<8)
-#define AUTOINDENT		(1<<9)
-#define VIEW_MODE		(1<<10)
-#define USE_MOUSE		(1<<11)
-#define USE_REGEXP		(1<<12)
-#define REGEXP_COMPILED		(1<<13)
-#define TEMP_OPT		(1<<14)
-#define CUT_TO_END		(1<<15)
-#define REVERSE_SEARCH		(1<<16)
-#define MULTIBUFFER		(1<<17)
-#define DOS_FILE		(1<<18)
-#define MAC_FILE		(1<<19)
-#define SMOOTHSCROLL		(1<<20)
-#define DISABLE_CURPOS		(1<<21)	/* Damn, we still need it. */
-#define REBIND_DELETE		(1<<22)
-#define NO_CONVERT		(1<<23)
-#define BACKUP_FILE		(1<<24)
-#define NO_RCFILE		(1<<25)
-#define COLOR_SYNTAX		(1<<26)
-#define PRESERVE		(1<<27)
-#define HISTORY_CHANGED		(1<<28)
-#define HISTORYLOG		(1<<29)
+#define CASE_SENSITIVE		(1<<1)
+#define MARK_ISSET		(1<<2)
+#define CONSTUPDATE		(1<<3)
+#define NO_HELP			(1<<4)
+#define NOFOLLOW_SYMLINKS	(1<<5)
+#define SUSPEND			(1<<6)
+#define NO_WRAP			(1<<7)
+#define AUTOINDENT		(1<<8)
+#define VIEW_MODE		(1<<9)
+#define USE_MOUSE		(1<<10)
+#define USE_REGEXP		(1<<11)
+#define TEMP_OPT		(1<<12)
+#define CUT_TO_END		(1<<13)
+#define REVERSE_SEARCH		(1<<14)
+#define MULTIBUFFER		(1<<15)
+#define DOS_FILE		(1<<16)
+#define MAC_FILE		(1<<17)
+#define SMOOTHSCROLL		(1<<18)
+#define DISABLE_CURPOS		(1<<19)	/* Damn, we still need it. */
+#define REBIND_DELETE		(1<<20)
+#define NO_CONVERT		(1<<21)
+#define BACKUP_FILE		(1<<22)
+#define NO_RCFILE		(1<<23)
+#define COLOR_SYNTAX		(1<<24)
+#define PRESERVE		(1<<25)
+#define HISTORY_CHANGED		(1<<26)
+#define HISTORYLOG		(1<<27)
+#define RESTRICTED		(1<<28)
+#define SMART_HOME		(1<<29)
+#define WHITESPACE_DISPLAY	(1<<30)
 
 /* Control key sequences, changing these would be very very bad. */
 #define NANO_CONTROL_SPACE 0
@@ -366,13 +353,13 @@ typedef struct historyheadtype {
 #define NANO_ALT_SPACE ' '
 
 /* Some semi-changeable keybindings; don't play with these unless you're
- * sure you know what you're doing. */
+ * sure you know what you're doing.  Assume ERR is defined as -1. */
 
 /* No key at all. */
-#define NANO_NO_KEY		-8
+#define NANO_NO_KEY		-2
 
-/* Special sentinel key used for search string history. */
-#define NANO_HISTORY_KEY	-9
+/* Special sentinel key used for search/replace history. */
+#define NANO_HISTORY_KEY	-3
 
 /* Normal keys. */
 #define NANO_INSERTFILE_KEY	NANO_CONTROL_R
@@ -442,6 +429,7 @@ typedef struct historyheadtype {
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
 #define NANO_PARABEGIN_KEY	NANO_CONTROL_W
 #define NANO_PARAEND_KEY	NANO_CONTROL_O
+#define NANO_FULLJUSTIFY_KEY	NANO_CONTROL_J
 #define NANO_VERBATIM_KEY	NANO_ALT_V
 
 #ifndef NANO_SMALL
@@ -463,6 +451,8 @@ typedef struct historyheadtype {
 #define TOGGLE_NOCONVERT_KEY	NANO_ALT_N
 #define TOGGLE_BACKUP_KEY	NANO_ALT_B
 #define TOGGLE_SYNTAX_KEY	NANO_ALT_Y
+#define TOGGLE_SMARTHOME_KEY	NANO_ALT_H
+#define TOGGLE_WHITESPACE_KEY	NANO_ALT_P
 #endif /* !NANO_SMALL */
 
 #define MAIN_VISIBLE 12
@@ -470,6 +460,14 @@ typedef struct historyheadtype {
 #define VIEW 1
 #define NOVIEW 0
 
+typedef enum {
+    JUSTIFY, BEGIN, END
+} justbegend;
+
+typedef enum {
+    UP, DOWN
+} updown;
+
 typedef enum {
     TOP, CENTER, NONE
 } topmidnone;
