@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.26 2004/03/30 04:17:10 dolorous Exp $ */
+/* $Id: proto.h,v 1.57 2004/06/04 18:18:17 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -46,7 +46,13 @@ extern int search_last_line;
 extern int search_offscreen;
 extern int currslen;
 
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+extern char *whitespace;
+#endif
+
 #ifndef DISABLE_JUSTIFY
+extern char *punct;
+extern char *brackets;
 extern char *quotestr;
 #endif
 
@@ -54,7 +60,7 @@ extern char *quotestr;
 extern char *backup_dir;
 #endif
 
-extern WINDOW *edit, *topwin, *bottomwin;
+extern WINDOW *topwin, *edit, *bottomwin;
 extern char *filename;
 extern struct stat originalfilestat;
 extern char *answer;
@@ -114,10 +120,10 @@ extern const shortcut *currshortcut;
 
 #ifdef HAVE_REGEX_H
 extern regex_t search_regexp;
-extern regmatch_t regmatches[10];  
+extern regmatch_t regmatches[10];
 #ifdef ENABLE_COLOR
 extern regex_t syntaxfile_regexp;
-extern regmatch_t synfilematches[1];  
+extern regmatch_t synfilematches[1];
 #endif /* ENABLE_COLOR */
 #endif /* HAVE_REGEX_H */
 
@@ -142,6 +148,7 @@ void update_color(void);
 #endif /* ENABLE_COLOR */
 
 /* Public functions in cut.c */
+void cutbuffer_reset(void);
 filestruct *get_cutbottom(void);
 void add_to_cutbuffer(filestruct *inptr, int allow_concat);
 void cut_marked_segment(void);
@@ -151,7 +158,8 @@ int do_uncut_text(void);
 /* Public functions in files.c */
 void load_file(int update);
 void new_file(void);
-filestruct *read_line(char *buf, filestruct *prev, int *line1ins, int len);
+filestruct *read_line(char *buf, filestruct *prev, int *line1ins, size_t
+	len);
 int read_file(FILE *f, const char *filename, int quiet);
 int open_file(const char *filename, int insert, int quiet);
 char *get_next_filename(const char *name);
@@ -187,8 +195,7 @@ void init_backup_dir(void);
 #endif
 int write_file(const char *name, int tmp, int append, int nonamechange);
 #ifndef NANO_SMALL
-int write_marked(const char *name, int tmp, int append, int
-	nonamechange);
+int write_marked(const char *name, int tmp, int append);
 #endif
 int do_writeout(int exiting);
 int do_writeout_void(void);
@@ -199,12 +206,10 @@ char **username_tab_completion(char *buf, int *num_matches);
 char **cwd_tab_completion(char *buf, int *num_matches);
 char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list);
 #endif
-#if !defined(DISABLE_BROWSER) || !defined(NANO_SMALL)
 const char *tail(const char *foo);
-#endif
 #ifndef DISABLE_BROWSER
 int diralphasort(const void *va, const void *vb);
-void free_charptrarray(char **array, int len);
+void free_charptrarray(char **array, size_t len);
 void striponedir(char *foo);
 int readable_dir(const char *path);
 char **browser_init(const char *path, int *longest, int *numents);
@@ -234,17 +239,21 @@ void thanks_for_all_the_fish(void);
 #endif
 
 /* Public functions in move.c */
+int do_first_line(void);
+int do_last_line(void);
 int do_home(void);
 int do_end(void);
 int do_page_up(void);
 int do_page_down(void);
 int do_up(void);
 int do_down(void);
-int do_left(void);
-int do_right(void);
+int do_left(int allow_update);
+int do_left_void(void);
+int do_right(int allow_update);
+int do_right_void(void);
 
 /* Public functions in nano.c */
-RETSIGTYPE finish(int sigage);
+void finish(void);
 void die(const char *msg, ...);
 void die_save_file(const char *die_filename);
 void die_too_small(void);
@@ -270,11 +279,8 @@ void print1opt(const char *shortflag, const char *longflag,
 		const char *desc);
 void usage(void);
 void version(void);
-void do_early_abort(void);
 int no_help(void);
-#if defined(DISABLE_JUSTIFY) || defined(DISABLE_SPELLER) || defined(DISABLE_HELP) || defined(NANO_SMALL)
-void nano_disabled_msg(void);
-#endif
+int nano_disabled_msg(void);
 #ifndef NANO_SMALL
 RETSIGTYPE cancel_fork(int signal);
 int open_pipe(const char *command);
@@ -291,23 +297,23 @@ int do_enter(void);
 #ifndef NANO_SMALL
 int do_next_word(void);
 int do_prev_word(void);
-#endif
 int do_mark(void);
+#endif
 #ifndef DISABLE_WRAPPING
 void wrap_reset(void);
 int do_wrap(filestruct *inptr);
 #endif
 #ifndef DISABLE_SPELLER
 int do_int_spell_fix(const char *word);
-char *do_int_speller(char *tempfile_name);
-char *do_alt_speller(char *tempfile_name);
-#endif
+const char *do_int_speller(char *tempfile_name);
+const char *do_alt_speller(char *tempfile_name);
 int do_spell(void);
+#endif
 #if !defined(DISABLE_WRAPPING) && !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
 size_t indent_length(const char *line);
 #endif
 #ifndef DISABLE_JUSTIFY
-int justify_format(int changes_allowed, filestruct *line, size_t skip);
+void justify_format(filestruct *line, size_t skip);
 #ifdef HAVE_REGEX_H
 size_t quote_length(const char *line, const regex_t *qreg);
 #else
@@ -318,20 +324,22 @@ size_t quote_length(const char *line);
 #else
 #  define IFREG(a, b) a
 #endif
-int quotes_match(const char *a_line, size_t a_quote,
-		IFREG(const char *b_line, const regex_t *qreg));
-size_t indents_match(const char *a_line, size_t a_indent,
-			const char *b_line, size_t b_indent);
-filestruct *backup_lines(filestruct *first_line, size_t par_len,
-			size_t quote_len);
+int quotes_match(const char *a_line, size_t a_quote, IFREG(const char
+	*b_line, const regex_t *qreg));
+size_t indents_match(const char *a_line, size_t a_indent, const char
+	*b_line, size_t b_indent);
+filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
+	quote_len);
 int breakable(const char *line, int goal);
 int break_line(const char *line, int goal, int force);
-int do_para_search(int search_type, size_t *quote, size_t *par, size_t
-	*indent, int do_refresh);
+int do_para_search(justbegend search_type, size_t *quote, size_t *par,
+	size_t *indent, int do_refresh);
 int do_para_begin(void);
 int do_para_end(void);
+int do_justify(int justify_all);
+int do_justify_void(void);
+int do_full_justify(void);
 #endif /* !DISABLE_JUSTIFY */
-int do_justify(void);
 int do_exit(void);
 void signal_init(void);
 RETSIGTYPE handle_hupterm(int signal);
@@ -341,10 +349,17 @@ RETSIGTYPE do_cont(int signal);
 void handle_sigwinch(int s);
 void allow_pending_sigwinch(int allow);
 #endif
-void print_numlock_warning(void);
 #ifndef NANO_SMALL
 void do_toggle(const toggle *which);
 #endif
+#if !defined(NANO_SMALL) || defined(USE_SLANG)
+void disable_signals(void);
+#endif
+#ifndef NANO_SMALL
+void enable_signals(void);
+#endif
+void disable_flow_control(void);
+void enable_flow_control(void);
 
 /* Public functions in rcfile.c */
 #ifdef ENABLE_NANORC
@@ -377,7 +392,9 @@ int is_whole_word(int curr_pos, const char *datastr, const char
 int findnextstr(int can_display_wrap, int wholeword, const filestruct
 	*begin, size_t beginx, const char *needle, int no_sameline);
 int do_search(void);
+#ifndef NANO_SMALL
 int do_research(void);
+#endif
 void replace_abort(void);
 #ifdef HAVE_REGEX_H
 int replace_regexp(char *string, int create_flag);
@@ -403,7 +420,7 @@ char *get_history_newer(historyheadtype *h);
 char *get_history_completion(historyheadtype *h, char *s);
 void free_history(historyheadtype *h);
 #ifdef ENABLE_NANORC
-void load_history(void);  
+void load_history(void);
 void save_history(void);
 #endif
 #endif
@@ -416,6 +433,9 @@ int regexec_safe(const regex_t *preg, const char *string, size_t nmatch,
 #endif
 int regexp_bol_or_eol(const regex_t *preg, const char *string);
 #endif
+#ifndef HAVE_ISBLANK
+int is_blank_char(int c);
+#endif
 int is_cntrl_char(int c);
 int num_of_digits(int n);
 void align(char **strp);
@@ -428,13 +448,18 @@ int nstricmp(const char *s1, const char *s2);
 #ifndef HAVE_STRNCASECMP
 int nstrnicmp(const char *s1, const char *s2, size_t n);
 #endif
+#ifndef HAVE_STRCASESTR
+const char *nstristr(const char *haystack, const char *needle);
+#endif
 #ifndef NANO_SMALL
-const char *revstrstr(const char *haystack, const char *needle,
-			const char *rev_start);
-const char *revstristr(const char *haystack, const char *needle,
-			const char *rev_start);
+const char *revstrstr(const char *haystack, const char *needle, const
+	char *rev_start);
+const char *revstristr(const char *haystack, const char *needle, const
+	char *rev_start);
+#endif
+#ifndef HAVE_STRNLEN
+size_t nstrnlen(const char *s, size_t maxlen);
 #endif
-const char *stristr(const char *haystack, const char *needle);
 const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start);
 void nperror(const char *s);
@@ -451,30 +476,45 @@ int check_wildcard_match(const char *text, const char *pattern);
 #endif
 
 /* Public functions in winio.c */
+#ifndef NANO_SMALL
+void reset_kbinput(void);
+#endif
 int get_kbinput(WINDOW *win, int *meta_key);
-int *get_verbatim_kbinput(WINDOW *win, int *kbinput_len, int
-	allow_ascii);
-int get_ignored_kbinput(WINDOW *win);
-int get_accepted_kbinput(WINDOW *win, int kbinput, int *meta_key);
-int get_ascii_kbinput(WINDOW *win, int kbinput);
-int get_escape_seq_kbinput(WINDOW *win, int *escape_seq, int
-	escape_seq_len);
+int get_translated_kbinput(int kbinput, int *es
+#ifndef NANO_SMALL
+	, int reset
+#endif
+	);
+int get_ascii_kbinput(int kbinput, size_t ascii_digits
+#ifndef NANO_SMALL
+	, int reset
+#endif
+	);
+int get_control_kbinput(int kbinput);
+int get_escape_seq_kbinput(int *escape_seq, size_t es_len, int
+	*ignore_seq);
 int get_escape_seq_abcd(int kbinput);
+int *get_verbatim_kbinput(WINDOW *win, int *verbatim_kbinput, size_t
+	*verbatim_len, int allow_ascii);
+int get_untranslated_kbinput(int kbinput, size_t position, int
+	allow_ascii
+#ifndef NANO_SMALL
+	, int reset
+#endif
+	);
+#ifndef DISABLE_MOUSE
 int get_mouseinput(int *mouse_x, int *mouse_y, int shortcut);
-int do_first_line(void);
-int do_last_line(void);
-int xpt(const filestruct *fileptr, int index);
+#endif
 size_t xplustabs(void);
 size_t actual_x(const char *str, size_t xplus);
 size_t strnlenpt(const char *buf, size_t size);
 size_t strlenpt(const char *buf);
-void blank_bottombars(void);
-void blank_bottomwin(void);
+void blank_titlebar(void);
 void blank_edit(void);
 void blank_statusbar(void);
-void blank_statusbar_refresh(void);
 void check_statblank(void);
-char *display_string(const char *buf, size_t start_col, int len);
+void blank_bottombars(void);
+char *display_string(const char *buf, size_t start_col, size_t len);
 void nanoget_repaint(const char *buf, const char *inputbuf, size_t x);
 int nanogetstr(int allowtabs, const char *buf, const char *def,
 #ifndef NANO_SMALL
@@ -485,24 +525,24 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 		, int *list
 #endif
 		);
-void set_modified(void);
 void titlebar(const char *path);
+void set_modified(void);
+void statusbar(const char *msg, ...);
 void bottombars(const shortcut *s);
-void onekey(const char *keystroke, const char *desc, int len);
+void onekey(const char *keystroke, const char *desc, size_t len);
 #ifndef NDEBUG
 int check_linenumbers(const filestruct *fileptr);
 #endif
 size_t get_page_start(size_t column);
 void reset_cursor(void);
-void add_marked_sameline(int begin, int end, filestruct *fileptr, int y,
-			 int virt_cur_x, int this_page);
-void edit_add(const filestruct *fileptr, const char *converted,
-		int yval, size_t start);
+void edit_add(const filestruct *fileptr, const char *converted, int
+	yval, size_t start);
 void update_line(const filestruct *fileptr, size_t index);
-void update_cursor(void);
-void center_cursor(void);
+int need_horizontal_update(int old_placewewant);
+int need_vertical_update(int old_placewewant);
+void edit_scroll(updown direction, int nlines);
+void edit_redraw(const filestruct *old_current, int old_pww);
 void edit_refresh(void);
-void edit_refresh_clearok(void);
 void edit_update(filestruct *fileptr, topmidnone location);
 int statusq(int allowtabs, const shortcut *s, const char *def,
 #ifndef NANO_SMALL
@@ -512,7 +552,6 @@ int statusq(int allowtabs, const shortcut *s, const char *def,
 int do_yesno(int all, const char *msg);
 int total_refresh(void);
 void display_main_list(void);
-void statusbar(const char *msg, ...);
 int do_cursorpos(int constant);
 int do_cursorpos_void(void);
 int line_len(const char *ptr);
