@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.9 2004/03/04 06:33:52 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.20 2004/05/30 04:19:27 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -31,6 +31,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <pwd.h>
+#include <ctype.h>
 #include <assert.h>
 #include "proto.h"
 #include "nano.h"
@@ -42,6 +43,9 @@ const static rcoption rcopts[] = {
     {"autoindent", AUTOINDENT},
     {"backup", BACKUP_FILE},
     {"backupdir", 0},
+#endif
+#ifndef DISABLE_JUSTIFY
+    {"brackets", 0},
 #endif
     {"const", CONSTUPDATE},
 #ifndef NANO_SMALL
@@ -71,14 +75,16 @@ const static rcoption rcopts[] = {
     {"operatingdir", 0},
 #endif
     {"preserve", PRESERVE},
-    {"rebinddelete", REBIND_DELETE},
 #ifndef DISABLE_JUSTIFY
+    {"punct", 0},
     {"quotestr", 0},
 #endif
+    {"rebinddelete", REBIND_DELETE},
 #ifdef HAVE_REGEX_H
     {"regexp", USE_REGEXP},
 #endif
 #ifndef NANO_SMALL
+    {"smarthome", SMART_HOME},
     {"smooth", SMOOTHSCROLL},
 #endif
 #ifndef DISABLE_SPELLER
@@ -88,6 +94,9 @@ const static rcoption rcopts[] = {
     {"tabsize", 0},
     {"tempfile", TEMP_OPT},
     {"view", VIEW_MODE},
+#ifndef NANO_SMALL
+    {"whitespace", 0},
+#endif
     {NULL, 0}
 };
 
@@ -131,7 +140,7 @@ void rcfile_msg(const char *msg, ...)
 /* Parse the next word from the string.  Returns NULL if we hit EOL. */
 char *parse_next_word(char *ptr)
 {
-    while (*ptr != ' ' && *ptr != '\t' && *ptr != '\n' && *ptr != '\0')
+    while (!isblank(*ptr) && *ptr != '\n' && *ptr != '\0')
 	ptr++;
 
     if (*ptr == '\0')
@@ -140,15 +149,16 @@ char *parse_next_word(char *ptr)
     /* Null terminate and advance ptr */
     *ptr++ = 0;
 
-    while (*ptr == ' ' || *ptr == '\t')
+    while (isblank(*ptr))
 	ptr++;
 
     return ptr;
 }
 
-/* The keywords operatingdir, backupdir, fill, tabsize, speller, and
- * quotestr take an argument when set.  Among these, operatingdir,
- * backupdir, speller, and quotestr have to allow tabs and spaces in the
+/* The keywords operatingdir, backupdir, fill, tabsize, speller,
+ * punct, brackets, quotestr, and whitespace take an argument when set.
+ * Among these, operatingdir, backupdir, speller, punct, brackets,
+ * quotestr, and whitespace have to allow tabs and spaces in the
  * argument.  Thus, if the next word starts with a ", we say it ends
  * with the last " of the line.  Otherwise, the word is interpreted as
  * usual.  That is so the arguments can contain "s too. */
@@ -179,7 +189,7 @@ char *parse_argument(char *ptr)
 	ptr = last_quote + 1;
     }
     if (ptr != NULL)
-	while (*ptr == ' ' || *ptr == '\t')
+	while (isblank(*ptr))
 	    ptr++;
     return ptr;
 }
@@ -237,7 +247,7 @@ char *parse_next_regex(char *ptr)
     /* Null terminate and advance ptr. */
     *ptr++ = '\0';
 
-    while (*ptr == ' ' || *ptr == '\t')
+    while (isblank(*ptr))
 	ptr++;
 
     return ptr;
@@ -326,7 +336,7 @@ void parse_syntax(char *ptr)
 	ptr = parse_next_regex(ptr);
 
 	newext = (exttype *)nmalloc(sizeof(exttype));
-	if (nregcomp(&newext->val, fileregptr, REG_NOSUB))
+	if (nregcomp(&newext->val, fileregptr, REG_NOSUB) != 0)
 	    free(newext);
 	else {
 	    if (endext == NULL)
@@ -413,7 +423,7 @@ void parse_colors(char *ptr)
 	newcolor = (colortype *)nmalloc(sizeof(colortype));
 	fgstr = ptr;
 	ptr = parse_next_regex(ptr);
-	if (nregcomp(&newcolor->start, fgstr, 0)) {
+	if (nregcomp(&newcolor->start, fgstr, 0) != 0) {
 	    free(newcolor);
 	    cancelled = 1;
 	} else {
@@ -464,7 +474,7 @@ void parse_colors(char *ptr)
 	    if (cancelled)
 		continue;
 	    newcolor->end = (regex_t *)nmalloc(sizeof(regex_t));
-	    if (nregcomp(newcolor->end, fgstr, 0)) {
+	    if (nregcomp(newcolor->end, fgstr, 0) != 0) {
 		free(newcolor->end);
 		newcolor->end = NULL;
 	    }
@@ -484,7 +494,7 @@ void parse_rcfile(FILE *rcstream)
     while (fgets(buf, 1023, rcstream) != 0) {
 	lineno++;
 	ptr = buf;
-	while (*ptr == ' ' || *ptr == '\t')
+	while (isblank(*ptr))
 	    ptr++;
 
 	if (*ptr == '\n' || *ptr == '\0')
@@ -538,7 +548,12 @@ void parse_rcfile(FILE *rcstream)
 #ifndef DISABLE_WRAPJUSTIFY
 				|| !strcasecmp(rcopts[i].name, "fill")
 #endif
+#ifndef NANO_SMALL
+				|| !strcasecmp(rcopts[i].name, "whitespace")
+#endif
 #ifndef DISABLE_JUSTIFY
+				|| !strcasecmp(rcopts[i].name, "punct")
+				|| !strcasecmp(rcopts[i].name, "brackets")
 				|| !strcasecmp(rcopts[i].name, "quotestr")
 #endif
 #ifndef NANO_SMALL
@@ -581,8 +596,35 @@ void parse_rcfile(FILE *rcstream)
 				    wrap_at = j;
 			    } else
 #endif
+#ifndef NANO_SMALL
+			    if (!strcasecmp(rcopts[i].name, "whitespace")) {
+				size_t ws_len;
+				whitespace = mallocstrcpy(NULL, option);
+				ws_len = strlen(whitespace);
+				if (ws_len != 2 || (ws_len == 2 && (is_cntrl_char(whitespace[0]) || is_cntrl_char(whitespace[1])))) {
+				    rcfile_error(_("Two non-control characters required"));
+				    free(whitespace);
+				    whitespace = NULL;
+				} else
+				    SET(WHITESPACE_DISPLAY);
+			    } else
+#endif
 #ifndef DISABLE_JUSTIFY
-			    if (!strcasecmp(rcopts[i].name, "quotestr"))
+			    if (!strcasecmp(rcopts[i].name, "punct")) {
+				punct = mallocstrcpy(NULL, option);
+				if (strchr(punct, '\t') != NULL || strchr(punct, ' ') != NULL) {
+				    rcfile_error(_("Non-tab and non-space characters required"));
+				    free(punct);
+				    punct = NULL;
+				}
+			    } else if (!strcasecmp(rcopts[i].name, "brackets")) {
+				brackets = mallocstrcpy(NULL, option);
+				if (strchr(brackets, '\t') != NULL || strchr(brackets, ' ') != NULL) {
+				    rcfile_error(_("Non-tab and non-space characters required"));
+				    free(brackets);
+				    brackets = NULL;
+				}
+			    } else if (!strcasecmp(rcopts[i].name, "quotestr"))
 				quotestr = mallocstrcpy(NULL, option);
 			    else
 #endif
@@ -596,7 +638,7 @@ void parse_rcfile(FILE *rcstream)
 				alt_speller = mallocstrcpy(NULL, option);
 			    else
 #endif
-			    {
+			    if (!strcasecmp(rcopts[i].name, "tabsize")) {
 				char *first_error;
 
 				/* Using strtol instead of atoi lets us
