@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.15 2004/03/15 20:26:30 dolorous Exp $ */
+/* $Id: search.c,v 1.24 2004/05/30 03:19:52 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -31,24 +31,39 @@
 #include "proto.h"
 #include "nano.h"
 
-/* Regular expression helper functions */
 
 #ifdef HAVE_REGEX_H
+static int regexp_compiled = FALSE;
+
+/* Regular expression helper functions. */
+
+/* Compile the given regular expression.  Return value 0 means the
+ * expression was invalid, and we wrote an error message on the status
+ * bar.  Return value 1 means success. */
 int regexp_init(const char *regexp)
 {
-    /* Hmm, perhaps we should check for whether regcomp returns successfully */
-    if (regcomp(&search_regexp, regexp, (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE) 
-		| REG_EXTENDED) != 0)
-	return 0;
+    int rc = regcomp(&search_regexp, regexp, REG_EXTENDED |
+	(ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE));
 
-    SET(REGEXP_COMPILED);
+    assert(!regexp_compiled);
+    if (rc != 0) {
+	size_t len = regerror(rc, &search_regexp, NULL, 0);
+	char *str = charalloc(len);
+
+	regerror(rc, &search_regexp, str, len);
+	statusbar(_("Bad regex \"%s\": %s"), regexp, str);
+	free(str);
+	return 0;
+    }
+
+    regexp_compiled = TRUE;
     return 1;
 }
 
 void regexp_cleanup(void)
 {
-    if (ISSET(REGEXP_COMPILED)) {
-	UNSET(REGEXP_COMPILED);
+    if (regexp_compiled) {
+	regexp_compiled = FALSE;
 	regfree(&search_regexp);
     }
 }
@@ -56,15 +71,18 @@ void regexp_cleanup(void)
 
 void not_found_msg(const char *str)
 {
-    if (strlen(str) <= COLS / 2)
-	statusbar(_("\"%s\" not found"), str);
-    else {
-	char *foo = mallocstrcpy(NULL, str);
+    char *disp;
+    int numchars;
+ 
+    assert(str != NULL);
 
-	foo[COLS / 2] = '\0';
-	statusbar(_("\"%s...\" not found"), foo);
-	free(foo);
-    }
+    disp = display_string(str, 0, (COLS / 2) + 1);
+    numchars = strnlen(disp, COLS / 2);
+
+    statusbar(_("\"%.*s%s\" not found"), numchars, disp,
+	disp[numchars] == '\0' ? "" : "...");
+
+    free(disp);
 }
 
 void search_abort(void)
@@ -150,9 +168,12 @@ int search_init(int replacing)
 #endif
 		"",
 
+#ifdef HAVE_REGEX_H
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
-	ISSET(USE_REGEXP) ? _(" [Regexp]") : "",
+	ISSET(USE_REGEXP) ? _(" [Regexp]") :
+#endif
+		"",
 
 #ifndef NANO_SMALL
 	/* This string is just a modifier for the search prompt; no
@@ -174,7 +195,6 @@ int search_init(int replacing)
     if (i == -1 || (i < 0 && last_search[0] == '\0') ||
 	    (!replacing && i == 0 && answer[0] == '\0')) {
 	statusbar(_("Search Cancelled"));
-	reset_cursor();
 #ifndef NANO_SMALL
 	search_history.current = search_history.next;
 #endif
@@ -184,21 +204,15 @@ int search_init(int replacing)
 	case -2:	/* It's the same string. */
 #ifdef HAVE_REGEX_H
 	    /* Since answer is "", use last_search! */
-	    if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0) {
-		statusbar(_("Invalid regex \"%s\""), last_search);
-		reset_cursor();
+	    if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
 		return -1;
-	    }
 #endif
 	    break;
 	case 0:		/* They entered something new. */
 	    last_replace[0] = '\0';
 #ifdef HAVE_REGEX_H
-	    if (ISSET(USE_REGEXP) && regexp_init(answer) == 0) {
-		statusbar(_("Invalid regex \"%s\""), answer);
-		reset_cursor();
+	    if (ISSET(USE_REGEXP) && regexp_init(answer) == 0)
 		return -1;
-	    }
 #endif
 	    break;
 #ifndef NANO_SMALL
@@ -311,7 +325,7 @@ int findnextstr(int can_display_wrap, int wholeword, const filestruct
 
 	/* Original start line reached. */
 	if (fileptr == begin)
-	    search_last_line = 1;
+	    search_last_line = TRUE;
 	rev_start = fileptr->data;
 #ifndef NANO_SMALL
 	if (ISSET(REVERSE_SEARCH))
@@ -347,9 +361,8 @@ int findnextstr(int can_display_wrap, int wholeword, const filestruct
 /* Search for a string. */
 int do_search(void)
 {
-    int i;
+    int old_pww = placewewant, i, fileptr_x = current_x, didfind;
     filestruct *fileptr = current;
-    int fileptr_x = current_x, didfind;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
@@ -382,10 +395,8 @@ int do_search(void)
 	update_history(&search_history, answer);
 #endif
 
-    search_last_line = 0;
+    search_last_line = FALSE;
     didfind = findnextstr(TRUE, FALSE, current, current_x, answer, FALSE);
-    edit_refresh();
-    placewewant = xplustabs();
 
     /* Check to see if there's only one occurrence of the string and
      * we're on it now. */
@@ -408,16 +419,19 @@ int do_search(void)
 #endif
     }
 
+    placewewant = xplustabs();
+    edit_redraw(fileptr, old_pww);
     search_abort();
 
     return 1;
 }
 
+#ifndef NANO_SMALL
 /* Search for the next string without prompting. */
 int do_research(void)
 {
+    int old_pww = placewewant, fileptr_x = current_x, didfind;
     filestruct *fileptr = current;
-    int fileptr_x = current_x, didfind;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
@@ -428,17 +442,12 @@ int do_research(void)
 
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
-	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0) {
-		statusbar(_("Invalid regex \"%s\""), last_search);
-		reset_cursor();
-		return -1;
-	    }
+	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
+	    return -1;
 #endif
 
-	search_last_line = 0;
+	search_last_line = FALSE;
 	didfind = findnextstr(TRUE, FALSE, current, current_x, last_search, FALSE);
-	edit_refresh();
-	placewewant = xplustabs();
 
 	/* Check to see if there's only one occurrence of the string and
 	 * we're on it now. */
@@ -463,10 +472,13 @@ int do_research(void)
     } else
         statusbar(_("No current search pattern"));
 
+    placewewant = xplustabs();
+    edit_redraw(fileptr, old_pww);
     search_abort();
 
     return 1;
 }
+#endif
 
 void replace_abort(void)
 {
@@ -533,7 +545,7 @@ char *replace_line(const char *needle)
     /* Calculate the size of the new line. */
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
- 	search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
+	search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
 	new_line_size = replace_regexp(NULL, 0);
     } else {
 #endif
@@ -575,12 +587,12 @@ char *replace_line(const char *needle)
 int do_replace_loop(const char *needle, const filestruct *real_current,
 	size_t *real_current_x, int wholewords)
 {
-    int replaceall = 0, numreplaced = -1;
-    const filestruct *current_save = current;
+    int old_pww = placewewant, replaceall = 0, numreplaced = -1;
     size_t current_x_save = current_x;
+    const filestruct *current_save = current;
 #ifdef HAVE_REGEX_H
     /* The starting-line match and bol/eol regex flags. */
-    int begin_line = 0, bol_or_eol = 0;
+    int begin_line = FALSE, bol_or_eol = FALSE;
 #endif
 #ifndef NANO_SMALL
     int mark_set = ISSET(MARK_ISSET);
@@ -617,12 +629,13 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	 * continue. */
 	else {
 	    if (current == real_current)
-		begin_line = 1;
-	    bol_or_eol = 0;
+		begin_line = TRUE;
+	    bol_or_eol = FALSE;
 	}
 #endif
 
-	edit_refresh();
+	if (!replaceall)
+	    edit_redraw(current_save, old_pww);
 
 #ifdef HAVE_REGEX_H
 	if (ISSET(USE_REGEXP))
@@ -660,7 +673,7 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	 * replace ("^", "$", or "^$"). */
 	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		needle))
-	    bol_or_eol = 1;
+	    bol_or_eol = TRUE;
 #endif
 
 	if (i > 0 || replaceall) {	/* Yes, replace it!!!! */
@@ -703,7 +716,15 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	    free(current->data);
 	    current->data = copy;
 
-	    edit_refresh();
+	    if (!replaceall) {
+#ifdef ENABLE_COLOR
+		if (ISSET(COLOR_SYNTAX))
+		    edit_refresh();
+		else
+#endif
+		    update_line(current, current_x);
+	    }
+
 	    set_modified();
 	    numreplaced++;
 	}
@@ -884,7 +905,7 @@ int do_find_bracket(void)
     char ch_under_cursor, wanted_ch;
     const char *pos, *brackets = "([{<>}])";
     char regexp_pat[] = "[  ]";
-    int flagsave, current_x_save, count = 1;
+    int old_pww = placewewant, current_x_save, flagsave, count = 1;
     filestruct *current_save;
 
     ch_under_cursor = current->data[current_x];
@@ -918,9 +939,9 @@ int do_find_bracket(void)
 
     regexp_init(regexp_pat);
     /* We constructed regexp_pat to be a valid expression. */
-    assert(ISSET(REGEXP_COMPILED));
+    assert(regexp_compiled);
 
-    search_last_line = 0;
+    search_last_line = FALSE;
     while (TRUE) {
 	if (findnextstr(FALSE, FALSE, current, current_x, regexp_pat, FALSE) != 0) {
 	    /* Found identical bracket. */
@@ -928,8 +949,8 @@ int do_find_bracket(void)
 		count++;
 	    /* Found complementary bracket. */
 	    else if (--count == 0) {
-		edit_refresh();
 		placewewant = xplustabs();
+		edit_redraw(current_save, old_pww);
 		break;
 	    }
 	} else {
