@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.10 2004/03/15 20:26:30 dolorous Exp $ */
+/* $Id: utils.c,v 1.13 2004/05/23 21:11:14 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -52,10 +52,20 @@ int regexp_bol_or_eol(const regex_t *preg, const char *string)
 }
 #endif /* HAVE_REGEX_H */
 
+#ifndef HAVE_ISBLANK
+/* This function is equivalent to isblank(). */
+int is_blank_char(int c)
+{
+    return (c == '\t' || c == ' ');
+}
+#endif
+
+/* This function is equivalent to iscntrl(), except in that it also
+ * handles control characters with their high bits set. */
 int is_cntrl_char(int c)
 {
     return (-128 <= c && c < -96) || (0 <= c && c < 32) ||
-		(127 <= c && c < 160);
+	(127 <= c && c < 160);
 }
 
 int num_of_digits(int n)
@@ -116,10 +126,12 @@ void sunder(char *str)
 int nstricmp(const char *s1, const char *s2)
 {
     assert(s1 != NULL && s2 != NULL);
+
     for (; *s1 != '\0' && *s2 != '\0'; s1++, s2++) {
 	if (tolower(*s1) != tolower(*s2))
 	    break;
     }
+
     return (tolower(*s1) - tolower(*s2));
 }
 #endif
@@ -129,23 +141,47 @@ int nstricmp(const char *s1, const char *s2)
 int nstrnicmp(const char *s1, const char *s2, size_t n)
 {
     assert(s1 != NULL && s2 != NULL);
+
     for (; n > 0 && *s1 != '\0' && *s2 != '\0'; n--, s1++, s2++) {
 	if (tolower(*s1) != tolower(*s2))
 	    break;
     }
+
     if (n > 0)
 	return (tolower(*s1) - tolower(*s2));
     else if (n == 0)
 	return 0;
-    else if (n < 0)
+    else
 	return -1;
 }
 #endif
 
+#ifndef HAVE_STRCASESTR
+/* This function is equivalent to strcasestr().  It was adapted from
+ * mutt's mutt_stristr() function. */
+const char *nstristr(const char *haystack, const char *needle)
+{
+    assert(haystack != NULL && needle != NULL);
+
+    for (; *haystack != '\0'; haystack++) {
+	const char *p = haystack;
+	const char *q = needle;
+
+	for (; tolower(*p) == tolower(*q) && *q != '\0'; p++, q++)
+	    ;
+
+	if (*q == '\0')
+	    return haystack;
+    }
+
+    return NULL;
+}
+#endif
+
 /* None of this is needed if we're using NANO_SMALL! */
 #ifndef NANO_SMALL
-const char *revstrstr(const char *haystack, const char *needle,
-			const char *rev_start)
+const char *revstrstr(const char *haystack, const char *needle, const
+	char *rev_start)
 {
     for (; rev_start >= haystack; rev_start--) {
 	const char *r, *q;
@@ -158,8 +194,8 @@ const char *revstrstr(const char *haystack, const char *needle,
     return NULL;
 }
 
-const char *revstristr(const char *haystack, const char *needle,
-			const char *rev_start)
+const char *revstristr(const char *haystack, const char *needle, const
+	char *rev_start)
 {
     for (; rev_start >= haystack; rev_start--) {
 	const char *r = rev_start, *q = needle;
@@ -173,26 +209,20 @@ const char *revstristr(const char *haystack, const char *needle,
 }
 #endif /* !NANO_SMALL */
 
-/* This is now mutt's version (called mutt_stristr) because it doesn't
- * use memory allocation to do a simple search (yuck). */
-const char *stristr(const char *haystack, const char *needle)
+#ifndef HAVE_STRNLEN
+/* This function is equivalent to strnlen(). */
+size_t nstrnlen(const char *s, size_t maxlen)
 {
-    const char *p, *q;
+    size_t n = 0;
 
-    if (haystack == NULL)
-	return NULL;
-    if (needle == NULL)
-	return haystack;
-    
-    while (*(p = haystack) != '\0') {
-	for (q = needle; *p != 0 && *q != 0 && tolower(*p) == tolower(*q); p++, q++)
-	    ;
-	if (*q == 0)
-	    return haystack;
-	haystack++;
-    }
-    return NULL;
+    assert(s != NULL);
+
+    for (; maxlen > 0 && *s != '\0'; maxlen--, n++, s++)
+	;
+
+    return n;
 }
+#endif
 
 /* If we are searching backwards, we will find the last match that
  * starts no later than start.  Otherwise we find the first match
@@ -253,7 +283,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
     else if (ISSET(REVERSE_SEARCH))
 	return revstristr(haystack, needle, start);
 #endif
-    return stristr(start, needle);
+    return strcasestr(start, needle);
 }
 
 /* This is a wrapper for the perror() function.  The wrapper takes care
